/*
*************************************************************************************
* Copyright 2011 Normation SAS
*************************************************************************************
*
* This file is part of Rudder.
* 
* Rudder is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* In accordance with the terms of section 7 (7. Additional Terms.) of
* the GNU General Public License version 3, the copyright holders add
* the following Additional permissions:
* Notwithstanding to the terms of section 5 (5. Conveying Modified Source
* Versions) and 6 (6. Conveying Non-Source Forms.) of the GNU General
* Public License version 3, when you create a Related Module, this
* Related Module is not considered as a part of the work and may be
* distributed under the license agreement of your choice.
* A "Related Module" means a set of sources files including their
* documentation that, without modification of the Source Code, enables
* supplementary functions or services in addition to those offered by
* the Software.
* 
* Rudder is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with Rudder.  If not, see <http://www.gnu.org/licenses/>.

*
*************************************************************************************
*/

package com.normation.rudder.services.policies

import com.normation.cfclerk.services.TechniquesLibraryUpdateNotification
import net.liftweb.common.Loggable
import com.normation.cfclerk.domain.TechniqueId
import com.normation.rudder.repository.WoDirectiveRepository
import net.liftweb.common._
import org.joda.time.DateTime
import com.normation.eventlog.EventActor
import com.normation.eventlog.ModificationId
import com.normation.rudder.repository.RoDirectiveRepository
import com.normation.utils.Control

class TechniqueAcceptationDatetimeUpdater(
    override val name:String
  , roActiveTechniqueRepo : RoDirectiveRepository
  , rwActiveTechniqueRepo : WoDirectiveRepository
) extends TechniquesLibraryUpdateNotification with Loggable {

    override def updatedTechniques(TechniqueIds:Seq[TechniqueId], modId: ModificationId, actor:EventActor, reason: Option[String]) : Box[Unit] = {
      val byNames = TechniqueIds.groupBy( _.name ).map { case (name,ids) =>
                      (name, ids.map( _.version ))
                    }.toSeq
      val acceptationDatetime = DateTime.now()

      Control.bestEffort(byNames) { case( name, versions ) =>
        roActiveTechniqueRepo.getActiveTechnique(name) match {
          case e:EmptyBox =>
            //OK, that policy package is not in the Active Technique Library, do nothing
            //log in case it was a real problem
            val error = e ?~! (s"The Technique with name '${name}' has been marked as updated in the Technique Library " +
                "but was not found in the Active Technique Library - it's expected if the technique was not added (or was removed) from Active Technique Library")
            logger.debug(error.messageChain)
            Full({})
          case Full(activeTechnique) =>
            logger.debug(s"Update acceptation datetime for technique '${activeTechnique.techniqueName}")
            val versionsMap = versions.map( v => (v,acceptationDatetime)).toMap
            rwActiveTechniqueRepo.setAcceptationDatetimes(activeTechnique.id, versionsMap, modId, actor, reason) match {
              case e:EmptyBox =>
                e ?~! s"Error when saving Active Technique ${activeTechnique.id.value} for technque ${activeTechnique.techniqueName}"
              case _ => Full({})
            }
        }
      }.map( _ => {})
    }

}
